
if [ -x /usr/bin/update-desktop-database ]; then
  /usr/bin/update-desktop-database /usr/share/applications >/dev/null 2>&1
fi

( cd usr/lib ; rm -rf libkwmf.so.8 )
( cd usr/lib ; ln -sf libkwmf.so.8.0.0 libkwmf.so.8 )
( cd usr/lib ; rm -rf libkritalibpaintop.so )
( cd usr/lib ; ln -sf libkritalibpaintop.so.8 libkritalibpaintop.so )
( cd usr/lib ; rm -rf libkomain.so.8 )
( cd usr/lib ; ln -sf libkomain.so.8.0.0 libkomain.so.8 )
( cd usr/lib ; rm -rf libkotext.so )
( cd usr/lib ; ln -sf libkotext.so.8 libkotext.so )
( cd usr/lib ; rm -rf libkowmf.so.8 )
( cd usr/lib ; ln -sf libkowmf.so.8.0.0 libkowmf.so.8 )
( cd usr/lib ; rm -rf libkexicore.so.8 )
( cd usr/lib ; ln -sf libkexicore.so.8.0.0 libkexicore.so.8 )
( cd usr/lib ; rm -rf libflake.so.8 )
( cd usr/lib ; ln -sf libflake.so.8.0.0 libflake.so.8 )
( cd usr/lib ; rm -rf libchartshapelib.so )
( cd usr/lib ; ln -sf libchartshapelib.so.8 libchartshapelib.so )
( cd usr/lib ; rm -rf libkexidatatable.so.8 )
( cd usr/lib ; ln -sf libkexidatatable.so.8.0.0 libkexidatatable.so.8 )
( cd usr/lib ; rm -rf libkpresenterprivate.so )
( cd usr/lib ; ln -sf libkpresenterprivate.so.8 libkpresenterprivate.so )
( cd usr/lib ; rm -rf libkritaimage.so.8 )
( cd usr/lib ; ln -sf libkritaimage.so.8.0.0 libkritaimage.so.8 )
( cd usr/lib ; rm -rf libkoreport.so )
( cd usr/lib ; ln -sf libkoreport.so.8 libkoreport.so )
( cd usr/lib ; rm -rf libkexirelationsview.so )
( cd usr/lib ; ln -sf libkexirelationsview.so.8 libkexirelationsview.so )
( cd usr/lib ; rm -rf libkplatoworkfactory.so.8 )
( cd usr/lib ; ln -sf libkplatoworkfactory.so.8.0.0 libkplatoworkfactory.so.8 )
( cd usr/lib ; rm -rf libkexiguiutils.so )
( cd usr/lib ; ln -sf libkexiguiutils.so.8 libkexiguiutils.so )
( cd usr/lib ; rm -rf libkformulalib.so )
( cd usr/lib ; ln -sf libkformulalib.so.8 libkformulalib.so )
( cd usr/lib ; rm -rf libkeximigrate.so )
( cd usr/lib ; ln -sf libkeximigrate.so.8 libkeximigrate.so )
( cd usr/lib ; rm -rf libkritaui.so )
( cd usr/lib ; ln -sf libkritaui.so.8 libkritaui.so )
( cd usr/lib ; rm -rf libkexiformutils.so )
( cd usr/lib ; ln -sf libkexiformutils.so.8 libkexiformutils.so )
( cd usr/lib ; rm -rf libchartshapelib.so.8 )
( cd usr/lib ; ln -sf libchartshapelib.so.8.0.0 libchartshapelib.so.8 )
( cd usr/lib ; rm -rf libkplatoui.so.8 )
( cd usr/lib ; ln -sf libkplatoui.so.8.0.0 libkplatoui.so.8 )
( cd usr/lib ; rm -rf libkformulaprivate.so.8 )
( cd usr/lib ; ln -sf libkformulaprivate.so.8.0.0 libkformulaprivate.so.8 )
( cd usr/lib ; rm -rf libkexidatatable.so )
( cd usr/lib ; ln -sf libkexidatatable.so.8 libkexidatatable.so )
( cd usr/lib ; rm -rf libkexidb.so.8 )
( cd usr/lib ; ln -sf libkexidb.so.8.0.0 libkexidb.so.8 )
( cd usr/lib ; rm -rf libkexiutils.so.8 )
( cd usr/lib ; ln -sf libkexiutils.so.8.0.0 libkexiutils.so.8 )
( cd usr/lib ; rm -rf libkarbonui.so )
( cd usr/lib ; ln -sf libkarbonui.so.8 libkarbonui.so )
( cd usr/lib ; rm -rf libkoodf.so )
( cd usr/lib ; ln -sf libkoodf.so.8 libkoodf.so )
( cd usr/lib ; rm -rf libmsooxml.so.8 )
( cd usr/lib ; ln -sf libmsooxml.so.8.0.0 libmsooxml.so.8 )
( cd usr/lib ; rm -rf libkformulalib.so.8 )
( cd usr/lib ; ln -sf libkformulalib.so.8.0.0 libkformulalib.so.8 )
( cd usr/lib ; rm -rf libkexiguiutils.so.8 )
( cd usr/lib ; ln -sf libkexiguiutils.so.8.0.0 libkexiguiutils.so.8 )
( cd usr/lib ; rm -rf libkplatoui.so )
( cd usr/lib ; ln -sf libkplatoui.so.8 libkplatoui.so )
( cd usr/lib ; rm -rf libkplatoworkapp.so.8 )
( cd usr/lib ; ln -sf libkplatoworkapp.so.8.0.0 libkplatoworkapp.so.8 )
( cd usr/lib ; rm -rf libflake.so )
( cd usr/lib ; ln -sf libflake.so.8 libflake.so )
( cd usr/lib ; rm -rf libkexiextendedwidgets.so )
( cd usr/lib ; ln -sf libkexiextendedwidgets.so.8 libkexiextendedwidgets.so )
( cd usr/lib ; rm -rf libkowv2.so.4 )
( cd usr/lib ; ln -sf libkowv2.so.4.0.1 libkowv2.so.4 )
( cd usr/lib ; rm -rf libkspreadcommon.so.8 )
( cd usr/lib ; ln -sf libkspreadcommon.so.8.0.0 libkspreadcommon.so.8 )
( cd usr/lib ; rm -rf libkoproperty.so.8 )
( cd usr/lib ; ln -sf libkoproperty.so.8.0.0 libkoproperty.so.8 )
( cd usr/lib ; rm -rf libkarboncommon.so )
( cd usr/lib ; ln -sf libkarboncommon.so.8 libkarboncommon.so )
( cd usr/lib ; rm -rf libkexiformutils.so.8 )
( cd usr/lib ; ln -sf libkexiformutils.so.8.0.0 libkexiformutils.so.8 )
( cd usr/lib ; rm -rf libkopageapp.so )
( cd usr/lib ; ln -sf libkopageapp.so.8 libkopageapp.so )
( cd usr/lib ; rm -rf libkexidb.so )
( cd usr/lib ; ln -sf libkexidb.so.8 libkexidb.so )
( cd usr/lib ; rm -rf libkwmf.so )
( cd usr/lib ; ln -sf libkwmf.so.8 libkwmf.so )
( cd usr/lib ; rm -rf libkwordexportfilters.so )
( cd usr/lib ; ln -sf libkwordexportfilters.so.8 libkwordexportfilters.so )
( cd usr/lib ; rm -rf libkoodf.so.8 )
( cd usr/lib ; ln -sf libkoodf.so.8.0.0 libkoodf.so.8 )
( cd usr/lib ; rm -rf libkotext.so.8 )
( cd usr/lib ; ln -sf libkotext.so.8.0.0 libkotext.so.8 )
( cd usr/lib ; rm -rf libkochart.so.8 )
( cd usr/lib ; ln -sf libkochart.so.8.0.0 libkochart.so.8 )
( cd usr/lib ; rm -rf libkplatokernel.so )
( cd usr/lib ; ln -sf libkplatokernel.so.8 libkplatokernel.so )
( cd usr/lib ; rm -rf libkwordprivate.so.8 )
( cd usr/lib ; ln -sf libkwordprivate.so.8.0.0 libkwordprivate.so.8 )
( cd usr/lib ; rm -rf libkformdesigner.so )
( cd usr/lib ; ln -sf libkformdesigner.so.8 libkformdesigner.so )
( cd usr/lib ; rm -rf libkritaimage.so )
( cd usr/lib ; ln -sf libkritaimage.so.8 libkritaimage.so )
( cd usr/lib ; rm -rf libkeximigrate.so.8 )
( cd usr/lib ; ln -sf libkeximigrate.so.8.0.0 libkeximigrate.so.8 )
( cd usr/lib ; rm -rf libkdchart.so )
( cd usr/lib ; ln -sf libkdchart.so.8 libkdchart.so )
( cd usr/lib ; rm -rf libkritalibbrush.so.8 )
( cd usr/lib ; ln -sf libkritalibbrush.so.8.0.0 libkritalibbrush.so.8 )
( cd usr/lib ; rm -rf libkexiextendedwidgets.so.8 )
( cd usr/lib ; ln -sf libkexiextendedwidgets.so.8.0.0 libkexiextendedwidgets.so.8 )
( cd usr/lib ; rm -rf libkokross.so.8 )
( cd usr/lib ; ln -sf libkokross.so.8.0.0 libkokross.so.8 )
( cd usr/lib ; rm -rf libkplatokernel.so.8 )
( cd usr/lib ; ln -sf libkplatokernel.so.8.0.0 libkplatokernel.so.8 )
( cd usr/lib ; rm -rf libpigmentcms.so.8 )
( cd usr/lib ; ln -sf libpigmentcms.so.8.0.0 libpigmentcms.so.8 )
( cd usr/lib ; rm -rf libkritalibpaintop.so.8 )
( cd usr/lib ; ln -sf libkritalibpaintop.so.8.0.0 libkritalibpaintop.so.8 )
( cd usr/lib ; rm -rf libkspreadodf.so.8 )
( cd usr/lib ; ln -sf libkspreadodf.so.8.0.0 libkspreadodf.so.8 )
( cd usr/lib ; rm -rf libkowidgets.so.8 )
( cd usr/lib ; ln -sf libkowidgets.so.8.0.0 libkowidgets.so.8 )
( cd usr/lib ; rm -rf libkritalibbrush.so )
( cd usr/lib ; ln -sf libkritalibbrush.so.8 libkritalibbrush.so )
( cd usr/lib ; rm -rf libkdchart.so.8 )
( cd usr/lib ; ln -sf libkdchart.so.8.0.0 libkdchart.so.8 )
( cd usr/lib ; rm -rf libkopageapp.so.8 )
( cd usr/lib ; ln -sf libkopageapp.so.8.0.0 libkopageapp.so.8 )
( cd usr/lib ; rm -rf libkexirelationsview.so.8 )
( cd usr/lib ; ln -sf libkexirelationsview.so.8.0.0 libkexirelationsview.so.8 )
( cd usr/lib ; rm -rf libkokross.so )
( cd usr/lib ; ln -sf libkokross.so.8 libkokross.so )
( cd usr/lib ; rm -rf libkoplugin.so.8 )
( cd usr/lib ; ln -sf libkoplugin.so.8.0.0 libkoplugin.so.8 )
( cd usr/lib ; rm -rf libpigmentcms.so )
( cd usr/lib ; ln -sf libpigmentcms.so.8 libpigmentcms.so )
( cd usr/lib ; rm -rf libkplatoworkfactory.so )
( cd usr/lib ; ln -sf libkplatoworkfactory.so.8 libkplatoworkfactory.so )
( cd usr/lib ; rm -rf libkarbonui.so.8 )
( cd usr/lib ; ln -sf libkarbonui.so.8.0.0 libkarbonui.so.8 )
( cd usr/lib ; rm -rf libkplatoprivate.so )
( cd usr/lib ; ln -sf libkplatoprivate.so.8 libkplatoprivate.so )
( cd usr/lib ; rm -rf libkplatomodels.so )
( cd usr/lib ; ln -sf libkplatomodels.so.8 libkplatomodels.so )
( cd usr/lib ; rm -rf libkeximain.so )
( cd usr/lib ; ln -sf libkeximain.so.8 libkeximain.so )
( cd usr/lib ; rm -rf libkoplugin.so )
( cd usr/lib ; ln -sf libkoplugin.so.8 libkoplugin.so )
( cd usr/lib ; rm -rf libkplatomodels.so.8 )
( cd usr/lib ; ln -sf libkplatomodels.so.8.0.0 libkplatomodels.so.8 )
( cd usr/lib ; rm -rf libkeximain.so.8 )
( cd usr/lib ; ln -sf libkeximain.so.8.0.0 libkeximain.so.8 )
( cd usr/lib ; rm -rf libkspreadcommon.so )
( cd usr/lib ; ln -sf libkspreadcommon.so.8 libkspreadcommon.so )
( cd usr/lib ; rm -rf libkwordprivate.so )
( cd usr/lib ; ln -sf libkwordprivate.so.8 libkwordprivate.so )
( cd usr/lib ; rm -rf libkplatoprivate.so.8 )
( cd usr/lib ; ln -sf libkplatoprivate.so.8.0.0 libkplatoprivate.so.8 )
( cd usr/lib ; rm -rf libkoproperty.so )
( cd usr/lib ; ln -sf libkoproperty.so.8 libkoproperty.so )
( cd usr/lib ; rm -rf libkformulaprivate.so )
( cd usr/lib ; ln -sf libkformulaprivate.so.8 libkformulaprivate.so )
( cd usr/lib ; rm -rf libkowidgets.so )
( cd usr/lib ; ln -sf libkowidgets.so.8 libkowidgets.so )
( cd usr/lib ; rm -rf libkexiutils.so )
( cd usr/lib ; ln -sf libkexiutils.so.8 libkexiutils.so )
( cd usr/lib ; rm -rf libkpresenterprivate.so.8 )
( cd usr/lib ; ln -sf libkpresenterprivate.so.8.0.0 libkpresenterprivate.so.8 )
( cd usr/lib ; rm -rf libkritaui.so.8 )
( cd usr/lib ; ln -sf libkritaui.so.8.0.0 libkritaui.so.8 )
( cd usr/lib ; rm -rf libkplatoworkapp.so )
( cd usr/lib ; ln -sf libkplatoworkapp.so.8 libkplatoworkapp.so )
( cd usr/lib ; rm -rf libmsooxml.so )
( cd usr/lib ; ln -sf libmsooxml.so.8 libmsooxml.so )
( cd usr/lib ; rm -rf libkwordexportfilters.so.8 )
( cd usr/lib ; ln -sf libkwordexportfilters.so.8.0.0 libkwordexportfilters.so.8 )
( cd usr/lib ; rm -rf libkowv2.so )
( cd usr/lib ; ln -sf libkowv2.so.4 libkowv2.so )
( cd usr/lib ; rm -rf libkowmf.so )
( cd usr/lib ; ln -sf libkowmf.so.8 libkowmf.so )
( cd usr/lib ; rm -rf libkarboncommon.so.8 )
( cd usr/lib ; ln -sf libkarboncommon.so.8.0.0 libkarboncommon.so.8 )
( cd usr/lib ; rm -rf libkomain.so )
( cd usr/lib ; ln -sf libkomain.so.8 libkomain.so )
( cd usr/lib ; rm -rf libkexicore.so )
( cd usr/lib ; ln -sf libkexicore.so.8 libkexicore.so )
( cd usr/lib ; rm -rf libkformdesigner.so.8 )
( cd usr/lib ; ln -sf libkformdesigner.so.8.0.0 libkformdesigner.so.8 )
( cd usr/lib ; rm -rf libkspreadodf.so )
( cd usr/lib ; ln -sf libkspreadodf.so.8 libkspreadodf.so )
( cd usr/lib ; rm -rf libkoreport.so.8 )
( cd usr/lib ; ln -sf libkoreport.so.8.0.0 libkoreport.so.8 )
( cd usr/lib ; rm -rf libkochart.so )
( cd usr/lib ; ln -sf libkochart.so.8 libkochart.so )
( cd usr/share/doc/HTML/en/koffice ; rm -rf common )
( cd usr/share/doc/HTML/en/koffice ; ln -sf /usr/share/doc/HTML/en/common common )
( cd usr/share/doc/HTML/en/kformula ; rm -rf common )
( cd usr/share/doc/HTML/en/kformula ; ln -sf /usr/share/doc/HTML/en/common common )
( cd usr/share/doc/HTML/en/kspread ; rm -rf common )
( cd usr/share/doc/HTML/en/kspread ; ln -sf /usr/share/doc/HTML/en/common common )
( cd usr/share/doc/HTML/en/kpresenter ; rm -rf common )
( cd usr/share/doc/HTML/en/kpresenter ; ln -sf /usr/share/doc/HTML/en/common common )
( cd usr/share/doc/HTML/en/karbon ; rm -rf common )
( cd usr/share/doc/HTML/en/karbon ; ln -sf /usr/share/doc/HTML/en/common common )
( cd usr/share/doc/HTML/en/kexi ; rm -rf common )
( cd usr/share/doc/HTML/en/kexi ; ln -sf /usr/share/doc/HTML/en/common common )
( cd usr/share/doc/HTML/en/thesaurus ; rm -rf common )
( cd usr/share/doc/HTML/en/thesaurus ; ln -sf /usr/share/doc/HTML/en/common common )
