<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd" >
<kpartgui name="KPlatoWork" version="0">
    <MenuBar>
        <Menu name="edit"><text>&amp;Edit</text>
           <Action name="package_remove_selected"/>
        </Menu>
<!--        <Menu name="settings"><text>&amp;Settings</text>
           <Action name="configure" group="settings_configure"/>
        </Menu>-->
    </MenuBar>
    <Menu name="taskstatus_popup">
        <Action name="task_progress"/>
        <Action name="task_description"/>
        <Action name="edit_sendpackage"/>
        <Action name="edit_packagesettings"/>
        <Action name="package_remove_current"/>
    </Menu>
    <Menu name="editdocument_popup">
        <Action name="view_document"/>
        <Action name="edit_document"/>
    </Menu>
    <Menu name="viewdocument_popup">
        <Action name="view_document"/>
    </Menu>

</kpartgui>
