<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui library="kritatonemapping" version="1">
<MenuBar>
    <Menu name="Layer"><text>&amp;Layer</text>
		<Action name="tonemapping"  group="layer_properties_merge"/>
    </Menu>
</MenuBar>
</kpartgui>
