<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui library="kritalayercompose" version="6">
<MenuBar>
    <Menu name="Layer"><text>Layer</text>
    <Menu name="LayerNew">
   	<Action name="layercompose"/>
    </Menu>
    </Menu>
</MenuBar>
</kpartgui>
